@@ -6,8 +6,8 @@ I/O device.
 __author__ = "Dennis van Gils"
 __authoremail__ = "vangils.dennis@gmail.com"
 __url__ = "https://github.com/Dennis-van-Gils/python-dvg-qdeviceio"
-__date__ = "17-07-2020"
-__version__ = "0.2.2"
+__date__ = "22-07-2020"
+__version__ = "0.3.0"
 # pylint: disable=protected-access
 
 from enum import IntEnum, unique
@@ -116,7 +116,6 @@ class QDeviceIO(QtCore.QObject):
                         DAQ_function               = DAQ_function,
                         DAQ_interval_ms            = 100,  # 100 ms -> 10 Hz
                         critical_not_alive_count   = 3,
-                        calc_DAQ_rate_every_N_iter = 10,
                         debug                      = debug,
                     )
 
@@ -295,9 +294,6 @@ class QDeviceIO(QtCore.QObject):
             dev (:obj:`object`):
                 Reference to a user-supplied *device* class instance containing
                 I/O methods.
-
-        Returns:
-            True if successful, False otherwise.
         """
         if not hasattr(dev, "name"):
             dev.name = "myDevice"
@@ -310,7 +306,6 @@ class QDeviceIO(QtCore.QObject):
 
         if type(self.dev) == self._NoDevice:
             self.dev = dev
-            return True
         else:
             pft(
                 "Device can be attached only once. Already attached to '%s'."
@@ -380,7 +375,7 @@ class QDeviceIO(QtCore.QObject):
         self,
         DAQ_priority=QtCore.QThread.InheritPriority,
         jobs_priority=QtCore.QThread.InheritPriority,
-    ):
+    ) -> bool:
         """Start the event loop of all of any created workers.
 
         Args:
@@ -411,7 +406,7 @@ class QDeviceIO(QtCore.QObject):
 
         return success
 
-    def start_worker_DAQ(self, priority=QtCore.QThread.InheritPriority):
+    def start_worker_DAQ(self, priority=QtCore.QThread.InheritPriority) -> bool:
         """Start the data acquisition with the device by starting the event loop
         of the :attr:`worker_DAQ` thread.
 
@@ -428,7 +423,7 @@ class QDeviceIO(QtCore.QObject):
                 "Did you forget to call 'create_worker_DAQ()' first?"
                 % self.dev.name
             )
-            sys.exit(404)
+            sys.exit(404)  # --> leaving
 
         elif not self.dev.is_alive:
             dprint(
@@ -436,10 +431,7 @@ class QDeviceIO(QtCore.QObject):
                 % self.dev.name,
                 ANSI.RED,
             )
-            self.worker_DAQ._started_okay = False
-
-        else:
-            self.worker_DAQ._started_okay = True
+            return False  # --> leaving
 
         if self.worker_DAQ.debug:
             tprint(
@@ -475,9 +467,11 @@ class QDeviceIO(QtCore.QObject):
             # to 'start_worker_DAQ()'.
             QtCore.QCoreApplication.processEvents()
 
-        return self.worker_DAQ._started_okay
+        return True
 
-    def start_worker_jobs(self, priority=QtCore.QThread.InheritPriority):
+    def start_worker_jobs(
+        self, priority=QtCore.QThread.InheritPriority
+    ) -> bool:
         """Start maintaining the jobs queue by starting the event loop of the
         :attr:`worker_jobs` thread.
 
@@ -494,7 +488,7 @@ class QDeviceIO(QtCore.QObject):
                 "Did you forget to call 'create_worker_jobs()' first?"
                 % self.dev.name
             )
-            sys.exit(404)
+            sys.exit(404)  # --> leaving
 
         elif not self.dev.is_alive:
             dprint(
@@ -502,10 +496,7 @@ class QDeviceIO(QtCore.QObject):
                 % self.dev.name,
                 ANSI.RED,
             )
-            self.worker_jobs._started_okay = False
-
-        else:
-            self.worker_jobs._started_okay = True
+            return False  # --> leaving
 
         if self.worker_jobs.debug:
             tprint(
@@ -532,13 +523,13 @@ class QDeviceIO(QtCore.QObject):
         # signal event.
         time.sleep(0.05)
 
-        return self.worker_jobs._started_okay
+        return True
 
     # --------------------------------------------------------------------------
     #   Quit workers
     # --------------------------------------------------------------------------
 
-    def quit(self):
+    def quit(self) -> bool:
         """Stop all of any running workers and close their respective threads.
 
         Returns:
@@ -546,30 +537,16 @@ class QDeviceIO(QtCore.QObject):
         """
         return self.quit_worker_DAQ() & self.quit_worker_jobs()
 
-    def quit_worker_DAQ(self):
+    def quit_worker_DAQ(self) -> bool:
         """Stop :attr:`worker_DAQ` and close its thread.
 
         Returns:
             True if successful, False otherwise.
         """
 
-        if self._thread_DAQ is None or self.worker_DAQ._started_okay is None:
-            # CASE: quit() without create()
-            #   _thread_DAQ == None
-            #   _started_okay = None
-            # CASE: quit() without start()
-            #   _thread_DAQ == QThread()
-            #   _started_okay = None
+        if self._thread_DAQ is None or not self.worker_DAQ._has_started:
             return True
 
-        # CASE: quit() with a device dead from the start
-        #   _thread_DAQ == QThread()
-        #   _started_okay = False
-
-        # CASE: quit() with a device alive from the start
-        #   _thread_DAQ == QThread()
-        #   _started_okay = True
-
         if self._thread_DAQ.isFinished():
             # CASE: Device has had a 'connection_lost' event during run-time,
             # which already stopped and closed the thread.
@@ -579,7 +556,7 @@ class QDeviceIO(QtCore.QObject):
             )
             return True
 
-        if not self.worker_DAQ._has_finished:
+        if not self.worker_DAQ._has_stopped:
             if self.worker_DAQ.debug:
                 tprint(
                     "Worker_DAQ  %s: stop requested..." % self.dev.name,
@@ -627,30 +604,16 @@ class QDeviceIO(QtCore.QObject):
             print("FAILED.\n", end="")  # pragma: no cover
             return False  # pragma: no cover
 
-    def quit_worker_jobs(self):
+    def quit_worker_jobs(self) -> bool:
         """Stop :attr:`worker_jobs` and close its thread.
 
         Returns:
             True if successful, False otherwise.
         """
 
-        if self._thread_jobs is None or self.worker_jobs._started_okay is None:
-            # CASE: quit() without create()
-            #   _thread_DAQ == None
-            #   _started_okay = None
-            # CASE: quit() without start()
-            #   _thread_DAQ == QThread()
-            #   _started_okay = None
+        if self._thread_jobs is None or not self.worker_jobs._has_started:
             return True
 
-        # CASE: quit() with a device dead from the start
-        #   _thread_DAQ == QThread()
-        #   _started_okay = False
-
-        # CASE: quit() with a device alive from the start
-        #   _thread_DAQ == QThread()
-        #   _started_okay = True
-
         if self._thread_jobs.isFinished():
             # CASE: Device has had a 'connection_lost' event during run-time,
             # which already stopped the worker and closed the thread.
@@ -660,7 +623,7 @@ class QDeviceIO(QtCore.QObject):
             )
             return True
 
-        if not self.worker_jobs._has_finished:
+        if not self.worker_jobs._has_stopped:
             if self.worker_jobs.debug:
                 tprint(
                     "Worker_jobs %s: stop requested..." % self.dev.name,
@@ -912,16 +875,6 @@ class Worker_DAQ(QtCore.QObject):
 
             Default: :const:`1`.
 
-            .. _`arg_calc_DAQ_rate_every_N_iter`:
-
-        calc_DAQ_rate_every_N_iter (:obj:`int`, optional):
-            To increase the accuracy of calculating the DAQ rate, it is advised
-            to average over several DAQ updates, i.e. iterations. It will take
-            at least *N* updates before :attr:`QDeviceIO.obtained_DAQ_rate_Hz`
-            will contain the calculated rate.
-
-            Default: :const:`10`.
-
         debug (:obj:`bool`, optional):
             Print debug info to the terminal? Warning: Slow! Do not leave on
             unintentionally.
@@ -951,10 +904,6 @@ class Worker_DAQ(QtCore.QObject):
         critical_not_alive_count (:obj:`int`):
             See the similarly named :ref:`initialization parameter
             <arg_critical_not_alive_count>`.
-
-        calc_DAQ_rate_every_N_iter (:obj:`int`):
-            See the similarly named :ref:`initialization parameter
-            <arg_calc_DAQ_rate_every_N_iter>`.
     """
 
     def __init__(
@@ -965,7 +914,6 @@ class Worker_DAQ(QtCore.QObject):
         DAQ_interval_ms=100,
         DAQ_timer_type=QtCore.Qt.PreciseTimer,
         critical_not_alive_count=1,
-        calc_DAQ_rate_every_N_iter=10,  # TODO: set default value to 'auto' and implement further down. When integer, take over that value.
         debug=False,
         **kwargs,
     ):
@@ -981,10 +929,17 @@ class Worker_DAQ(QtCore.QObject):
         self._DAQ_interval_ms = DAQ_interval_ms
         self._DAQ_timer_type = DAQ_timer_type
         self.critical_not_alive_count = critical_not_alive_count
-        self.calc_DAQ_rate_every_N_iter = calc_DAQ_rate_every_N_iter
 
-        self._started_okay = None
-        self._has_finished = False
+        self._has_started = False
+        self._has_stopped = False
+
+        # Keep track of the obtained DAQ interval and DAQ rate using
+        # QElapsedTimer (QET)
+        self._QET_interval = QtCore.QElapsedTimer()
+        self._QET_rate = QtCore.QElapsedTimer()
+        # Accumulates the number of DAQ updates passed since the previous DAQ
+        # rate evaluation
+        self._rate_accumulator = 0
 
         # Members specifically for INTERNAL_TIMER
         if self._DAQ_trigger == DAQ_TRIGGER.INTERNAL_TIMER:
@@ -993,11 +948,6 @@ class Worker_DAQ(QtCore.QObject):
             self._timer.setTimerType(DAQ_timer_type)
             self._timer.timeout.connect(self._perform_DAQ)
 
-            # TODO: create a special value, like string 'auto_1_Hz' to
-            # trigger below calculation
-            # self.calc_DAQ_rate_every_N_iter = max(
-            #        round(1e3/DAQ_interval_ms), 1)
-
         # Members specifically for SINGLE_SHOT_WAKE_UP
         elif self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             self._running = True
@@ -1012,12 +962,6 @@ class Worker_DAQ(QtCore.QObject):
             self._pause = None  # Will be set at init of '_do_work()' when 'start_worker_DAQ()' is called
             self._paused = None  # Will be set at init of '_do_work()' when 'start_worker_DAQ()' is called
 
-        # QElapsedTimer (QET) to keep track of DAQ interval and DAQ rate
-        self._QET_DAQ = QtCore.QElapsedTimer()
-        self._QET_DAQ.start()
-        self._prev_tick_DAQ_update = 0
-        self._prev_tick_DAQ_rate = 0
-
         if self.debug:
             tprint(
                 "Worker_DAQ  %s: init @ thread %s"
@@ -1038,12 +982,13 @@ class Worker_DAQ(QtCore.QObject):
 
             if self.debug:
                 tprint(
-                    "Worker_DAQ  %s: start confirmed" % self.dev.name,
+                    "Worker_DAQ  %s: has started" % self.dev.name,
                     self.debug_color,
                 )
 
             # Send confirmation
             self.qdev._qwc_worker_DAQ_started.wakeAll()
+            self._has_started = True
 
         if self.debug:
             tprint(
@@ -1080,7 +1025,7 @@ class Worker_DAQ(QtCore.QObject):
 
                 if self.debug:
                     tprint(
-                        "Worker_DAQ  %s: wake confirmed" % self.dev.name,
+                        "Worker_DAQ  %s: has woken up" % self.dev.name,
                         self.debug_color,
                     )
 
@@ -1093,18 +1038,16 @@ class Worker_DAQ(QtCore.QObject):
 
             if self.debug:
                 tprint(
-                    "Worker_DAQ  %s: stop confirmed" % self.dev.name,
+                    "Worker_DAQ  %s: has stopped" % self.dev.name,
                     self.debug_color,
                 )
 
             # Wait a tiny amount for the other thread to have entered the
             # QWaitCondition lock, before giving a wakingAll().
             QtCore.QTimer.singleShot(
-                100,
-                lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
+                100, self.qdev._qwc_worker_DAQ_stopped.wakeAll,
             )
-
-            self._has_finished = True
+            self._has_stopped = True
 
         # CONTINUOUS
         elif self._DAQ_trigger == DAQ_TRIGGER.CONTINUOUS:
@@ -1129,8 +1072,7 @@ class Worker_DAQ(QtCore.QObject):
                     if self._pause != self._paused:
                         if self.debug and not init:
                             tprint(
-                                "Worker_DAQ  %s: pause confirmed"
-                                % self.dev.name,
+                                "Worker_DAQ  %s: has paused" % self.dev.name,
                                 self.debug_color,
                             )
                         self.qdev.signal_DAQ_paused.emit()
@@ -1142,8 +1084,7 @@ class Worker_DAQ(QtCore.QObject):
                     if self._pause != self._paused:
                         if self.debug:
                             tprint(
-                                "Worker_DAQ  %s: unpause confirmed"
-                                % self.dev.name,
+                                "Worker_DAQ  %s: has unpaused" % self.dev.name,
                                 self.debug_color,
                             )
                         self._paused = False
@@ -1152,7 +1093,7 @@ class Worker_DAQ(QtCore.QObject):
 
             if self.debug:
                 tprint(
-                    "Worker_DAQ  %s: stop confirmed" % self.dev.name,
+                    "Worker_DAQ  %s: has stopped" % self.dev.name,
                     self.debug_color,
                 )
 
@@ -1160,17 +1101,15 @@ class Worker_DAQ(QtCore.QObject):
             # in a different thread than this one, to have entered the
             # QWaitCondition lock, before giving a wakingAll().
             QtCore.QTimer.singleShot(
-                100,
-                lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
+                100, self.qdev._qwc_worker_DAQ_stopped.wakeAll,
             )
-
-            self._has_finished = True
+            self._has_stopped = True
 
     @_coverage_resolve_trace
     @QtCore.pyqtSlot()
     def _perform_DAQ(self):
-        if not self._started_okay:
-            return
+        # if not self._has_started:  # TODO: Might be obsolete. Scheduled for remove.
+        #    return
 
         locker = QtCore.QMutexLocker(self.dev.mutex)
         self.qdev.update_counter_DAQ += 1
@@ -1182,40 +1121,6 @@ class Worker_DAQ(QtCore.QObject):
                 self.debug_color,
             )
 
-        # Keep track of the obtained DAQ update interval
-        now = self._QET_DAQ.elapsed()
-        if self.qdev.update_counter_DAQ > 1:
-            self.qdev.obtained_DAQ_interval_ms = (
-                now - self._prev_tick_DAQ_update
-            )
-        self._prev_tick_DAQ_update = now
-
-        # Keep track of the obtained DAQ rate
-        if self.qdev.update_counter_DAQ % self.calc_DAQ_rate_every_N_iter == 0:
-            try:
-                self.qdev.obtained_DAQ_rate_Hz = (
-                    self.calc_DAQ_rate_every_N_iter
-                    / (now - self._prev_tick_DAQ_rate)
-                    * 1e3
-                )
-            except ZeroDivisionError:  # pragma: no cover
-                self.qdev.obtained_DAQ_rate_Hz = np.nan  # pragma: no cover
-            self._prev_tick_DAQ_rate = now
-
-        # Check the not alive counter
-        if self.qdev.not_alive_counter_DAQ >= self.critical_not_alive_count:
-            dprint(
-                "Worker_DAQ  %s: Lost connection to device." % self.dev.name,
-                ANSI.RED,
-            )
-            self.dev.is_alive = False
-
-            locker.unlock()
-            self._stop()
-            # self.qdev.signal_DAQ_updated.emit()  # TODO: Was uncommented in the prototype version. Check if we can safely remove this line.
-            self.qdev.signal_connection_lost.emit()
-            return
-
         # ----------------------------------
         #   User-supplied DAQ function
         # ----------------------------------
@@ -1249,6 +1154,41 @@ class Worker_DAQ(QtCore.QObject):
             )
 
         locker.unlock()
+
+        # Keep track of the obtained DAQ interval and DAQ rate
+        if not self._QET_interval.isValid():
+            self._QET_interval.start()
+            self._QET_rate.start()
+        else:
+            # Obtained DAQ interval
+            self.qdev.obtained_DAQ_interval_ms = self._QET_interval.restart()
+
+            # Obtained DAQ rate
+            self._rate_accumulator += 1
+            dT = self._QET_rate.elapsed()
+
+            if dT >= 1000:  # Evaluate every N elapsed milliseconds. Hard-coded.
+                self._QET_rate.restart()
+                try:
+                    self.qdev.obtained_DAQ_rate_Hz = (
+                        self._rate_accumulator / dT * 1e3
+                    )
+                except ZeroDivisionError:  # pragma: no cover
+                    self.qdev.obtained_DAQ_rate_Hz = np.nan  # pragma: no cover
+
+                self._rate_accumulator = 0
+
+        # Check the not alive counter
+        if self.qdev.not_alive_counter_DAQ >= self.critical_not_alive_count:
+            dprint(
+                "Worker_DAQ  %s: Lost connection to device." % self.dev.name,
+                ANSI.RED,
+            )
+            self.dev.is_alive = False
+            self._stop()
+            self.qdev.signal_connection_lost.emit()
+            return
+
         self.qdev.signal_DAQ_updated.emit()
 
     @QtCore.pyqtSlot()
@@ -1264,18 +1204,16 @@ class Worker_DAQ(QtCore.QObject):
 
             if self.debug:
                 tprint(
-                    "Worker_DAQ  %s: stop confirmed" % self.dev.name,
+                    "Worker_DAQ  %s: has stopped" % self.dev.name,
                     self.debug_color,
                 )
 
             # Wait a tiny amount for the other thread to have entered the
             # QWaitCondition lock, before giving a wakingAll().
             QtCore.QTimer.singleShot(
-                100,
-                lambda: self.qdev._qwc_worker_DAQ_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
+                100, self.qdev._qwc_worker_DAQ_stopped.wakeAll,
             )
-
-            self._has_finished = True
+            self._has_stopped = True
 
         elif self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             self._running = False
@@ -1340,7 +1278,7 @@ class Worker_DAQ(QtCore.QObject):
         if self._DAQ_trigger == DAQ_TRIGGER.SINGLE_SHOT_WAKE_UP:
             if self.debug:
                 tprint(
-                    "Worker_DAQ  %s: wake requested..." % self.dev.name,
+                    "Worker_DAQ  %s: wake-up requested..." % self.dev.name,
                     ANSI.WHITE,
                 )
 
@@ -1476,8 +1414,8 @@ class Worker_jobs(QtCore.QObject):
         self.dev = None if qdev is None else qdev.dev
 
         self.jobs_function = jobs_function
-        self._started_okay = None
-        self._has_finished = False
+        self._has_started = False
+        self._has_stopped = False
 
         self._running = True
         self._qwc = QtCore.QWaitCondition()
@@ -1509,12 +1447,13 @@ class Worker_jobs(QtCore.QObject):
 
             if self.debug:
                 tprint(
-                    "Worker_jobs %s: start confirmed" % self.dev.name,
+                    "Worker_jobs %s: has started" % self.dev.name,
                     self.debug_color,
                 )
 
             # Send confirmation
             self.qdev._qwc_worker_jobs_started.wakeAll()
+            self._has_started = True
 
         if self.debug:
             tprint(
@@ -1544,7 +1483,7 @@ class Worker_jobs(QtCore.QObject):
 
             if self.debug:
                 tprint(
-                    "Worker_jobs %s: wake confirmed" % self.dev.name,
+                    "Worker_jobs %s: has woken up" % self.dev.name,
                     self.debug_color,
                 )
 
@@ -1557,24 +1496,21 @@ class Worker_jobs(QtCore.QObject):
 
         if self.debug:
             tprint(
-                "Worker_jobs %s: stop confirmed" % self.dev.name,
-                self.debug_color,
+                "Worker_jobs %s: has stopped" % self.dev.name, self.debug_color,
             )
 
         # Wait a tiny amount for the other thread to have entered the
         # QWaitCondition lock, before giving a wakingAll().
         QtCore.QTimer.singleShot(
-            100,
-            lambda: self.qdev._qwc_worker_jobs_stopped.wakeAll(),  # pylint: disable=unnecessary-lambda
+            100, self.qdev._qwc_worker_jobs_stopped.wakeAll,
         )
-
-        self._has_finished = True
+        self._has_stopped = True
 
     @_coverage_resolve_trace
     @QtCore.pyqtSlot()
     def _perform_jobs(self):
-        if not self._started_okay:
-            return
+        # if not self._has_started:  # TODO: Might be obsolete. Scheduled for remove.
+        #    return
 
         locker = QtCore.QMutexLocker(self.dev.mutex)
         self.qdev.update_counter_jobs += 1
@@ -1684,7 +1620,8 @@ class Worker_jobs(QtCore.QObject):
         """
         if self.debug:
             tprint(
-                "Worker_jobs %s: wake requested..." % self.dev.name, ANSI.WHITE,
+                "Worker_jobs %s: wake-up requested..." % self.dev.name,
+                ANSI.WHITE,
             )
 
         self._qwc.wakeAll()
